package hr.webtech.pay2.example;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Toast;

import ba.leftor.android.pikpay.BuildConfig;
import ba.leftor.android.pikpay.R;
import hr.webtech.pay2.api.PgwModule;
import hr.webtech.pay2.data.AddPaymentMethodModel;
import hr.webtech.pay2.data.ApiError;
import hr.webtech.pay2.data.Customer;
import hr.webtech.pay2.data.Merchant;
import hr.webtech.pay2.data.PaymentMethod;
import hr.webtech.pay2.example.data.MerchantProvider;
import hr.webtech.pay2.view.AddPaymentMethodLifecycleAdapter;
import hr.webtech.pay2.view.AddPaymentMethodLifecycleDelegate;
import hr.webtech.pay2.view.WebtehWebView;

/**
 * Example activity with {@link #webtehWebView}.
 * <p>
 */
public class AddPaymentMethodActivity extends Activity {

    private static final String TAG = AddPaymentMethodActivity.class.getSimpleName();
    public static final String BNLD_MERCHANT = "bnld_merchant";
    private WebtehWebView webtehWebView;
    private Merchant merchant;

    public static Intent createIntent(Context context, Merchant merchant) {
        final Intent intent = new Intent(context, AddPaymentMethodActivity.class);
        intent.putExtra(BNLD_MERCHANT, merchant);
        return intent;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_webteh_web_view_example);

        final Intent intent = getIntent();

        final Merchant merchantParcelable = intent.getParcelableExtra(BNLD_MERCHANT);

        if (merchantParcelable == null) {
            this.merchant = MerchantProvider.getAllMerchants().get(0);
        } else {
            this.merchant = merchantParcelable;
        }

        this.webtehWebView = (WebtehWebView) findViewById(R.id.webView);
        View progressBar = findViewById(R.id.progressBar);

        PgwModule pgwModule = new PgwModule(this.merchant.getAuthenticationToken(), BuildConfig.DEBUG);

        final String orderNumber = String.valueOf(System.currentTimeMillis());

        final AddPaymentMethodLifecycleAdapter addPaymentMethodLifecycleAdapter = new AddPaymentMethodLifecycleAdapter(orderNumber,
                pgwModule.getApi(),
                merchant.getMerchantKey(),
                new AddPaymentMethodLifecycleDelegateImpl(progressBar, getApplicationContext())
        );

        Log.d(TAG, "onCreate: " + getApplicationContext().getResources().getConfiguration().locale);

        webtehWebView.initialize(BuildConfig.DEBUG, 60000)
                .addPaymentMethod(addPaymentMethodLifecycleAdapter, createAddPaymentMethod(orderNumber));
    }

    private AddPaymentMethodModel createAddPaymentMethod(String orderNumber) {

        Customer customer = new Customer(
                "John Smith",
                "test@test.com",
                "Address",
                "+1-202-555-0138",
                "Sarajevo",
                "test@test.com"
        );

        return new AddPaymentMethodModel(
                orderNumber,
                "order info",
                "009",
                "add_payment_method",
                merchant, customer);
    }

    static class AddPaymentMethodLifecycleDelegateImpl implements AddPaymentMethodLifecycleDelegate {

        private final View progressBar;
        private final Context context;

        AddPaymentMethodLifecycleDelegateImpl(View progressBar, Context context) {
            this.progressBar = progressBar;
            this.context = context;
        }

        @Override
        public void showProgress() {
            progressBar.setVisibility(View.VISIBLE);
        }

        @Override
        public void hideProgress() {
            progressBar.setVisibility(View.INVISIBLE);
        }

        @Override
        public void addPaymentMethodSuccess(PaymentMethod paymentMethod) {
            Toast.makeText(context, paymentMethod.toString(), Toast.LENGTH_LONG).show();
        }

        @Override
        public void backPressedTransactionInProgress() {
            Log.d(TAG, "backPressedTransactionInProgress");
        }

        @Override
        public void threeDsSubmitted() {
            Log.d(TAG, "threeDsSubmitted");
        }

        @Override
        public void threeDsLoaded() {
            Log.d(TAG, "threeDsLoaded");
        }

        @Override
        public void handleTransactionError(ApiError apiError) {
            Log.e(TAG, "handleTransactionError: " + apiError, apiError);
        }

        @Override
        public void stopWebViewLoad() {
            Log.d(TAG, "stopWebViewLoad");
        }

        @Override
        public void logTransactionInfo(String loggingInfo) {
            Log.d(TAG, "logTransactionInfo: " + loggingInfo);
        }
    }

    /**
     * To prevent stopping ongoing transaction add code bellow in {@link #onBackPressed()}
     */
    @Override
    public void onBackPressed() {
        if (webtehWebView.isBackAllowed()) {
            super.onBackPressed();
        }
    }

    /**
     * For best behaviour invoke {@link WebtehWebView#onDestroy()} in appropriate lifecycle event
     */
    @Override
    protected void onDestroy() {
        super.onDestroy();
        webtehWebView.onDestroy();
    }

}
