package hr.webtech.pay2.example;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

import ba.leftor.android.pikpay.BuildConfig;
import ba.leftor.android.pikpay.R;
import hr.webtech.pay2.api.PgwApi;
import hr.webtech.pay2.api.PgwModule;
import hr.webtech.pay2.data.AddCardModel;
import hr.webtech.pay2.data.Charge;
import hr.webtech.pay2.data.Merchant;
import hr.webtech.pay2.data.PaymentMethod;
import hr.webtech.pay2.data.PaymentMethodResponse;
import hr.webtech.pay2.data.TransactionSuccess;
import rx.Subscription;
import rx.android.schedulers.AndroidSchedulers;
import rx.functions.Action1;
import rx.schedulers.Schedulers;
import rx.subscriptions.CompositeSubscription;

import static hr.webtech.pay2.example.WebtehWebViewExampleActivity.BNLD_CHARGE_EXAMPLE;
import static hr.webtech.pay2.example.WebtehWebViewExampleActivity.BNLD_MERCHANT;

public class PickPaymentMethodActivity extends AppCompatActivity {

    private PaymentMethodAdapter adapter;
    private Merchant merchant;
    private PgwApi api;
    private boolean chargeExample;

    private CompositeSubscription compositeSubscription = new CompositeSubscription();

    public static Intent createIntent(Context context, Merchant merchant) {
        return createIntent(context, merchant, false);
    }

    public static Intent createIntent(Context context, Merchant merchant, boolean chargeExample) {
        Intent intent = new Intent(context, PickPaymentMethodActivity.class);
        intent.putExtra(BNLD_MERCHANT, merchant);
        intent.putExtra(BNLD_CHARGE_EXAMPLE, chargeExample);
        return intent;
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_pick_payment_method);

        final Intent intent = getIntent();
        this.chargeExample = intent.getBooleanExtra(BNLD_CHARGE_EXAMPLE, false);

        final ListView listview = findViewById(R.id.listview);

        TextView title = findViewById(R.id.pick_payment_method);

        this.merchant = intent.getParcelableExtra(BNLD_MERCHANT);
        PgwModule pgwModule = new PgwModule(merchant.getAuthenticationToken(), BuildConfig.DEBUG);

        title.setText(String.format("Pick payment method, merchant: %s", merchant.getMerchantId()));

        this.api = pgwModule.getApi();

        final List<PaymentMethod> list = new ArrayList<>();

        this.adapter = new PaymentMethodAdapter(this,
                list, new Action1<PaymentMethod>() {
            @Override
            public void call(PaymentMethod paymentMethod) {
                goToPayment(paymentMethod);
            }
        });
        listview.setAdapter(adapter);

        final Subscription subscribe = api.paymentMethods("test@test.com")
                .subscribeOn(Schedulers.io())
                .observeOn(AndroidSchedulers.mainThread())
                .subscribe(new Action1<PaymentMethodResponse>() {
                    @Override
                    public void call(PaymentMethodResponse paymentMethodResponse) {
                        adapter.setValues(paymentMethodResponse.getPaymentMethods());
                    }
                });

        compositeSubscription.add(subscribe);
    }

    private void goToPayment(PaymentMethod paymentMethod) {
        if (chargeExample) {
            api.charge(Charge.create(
                    paymentMethod,
                    "100",
                    AddCardModel.Currency.BAM,
                    UUID.randomUUID().toString(),
                    "random order info - pay without cvv"
            ))
                    .subscribeOn(Schedulers.io())
                    .observeOn(AndroidSchedulers.mainThread())
                    .subscribe(new Action1<TransactionSuccess>() {
                        @Override
                        public void call(TransactionSuccess transactionSuccess) {
                            Toast.makeText(PickPaymentMethodActivity.this, transactionSuccess.toString(), Toast.LENGTH_SHORT).show();
                        }
                    }, new Action1<Throwable>() {
                        @Override
                        public void call(Throwable throwable) {
                            Toast.makeText(PickPaymentMethodActivity.this, throwable.getMessage(), Toast.LENGTH_SHORT).show();
                        }
                    });
        } else {
            final Intent intent = WebtehWebViewExampleActivity.createIntent(this, true, merchant, paymentMethod);
            startActivity(intent);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        compositeSubscription.clear();
    }
}
