package hr.webtech.pay2.api;

import java.util.Date;

import hr.webtech.pay2.data.Charge;
import hr.webtech.pay2.data.DeletePaymentResponse;
import hr.webtech.pay2.data.PaymentMethodResponse;
import hr.webtech.pay2.data.ShowOrderResponse;
import hr.webtech.pay2.data.Transaction;
import hr.webtech.pay2.data.TransactionError;
import hr.webtech.pay2.data.TransactionErrorDetailed;
import hr.webtech.pay2.data.TransactionSuccess;
import hr.webtech.pay2.data.UpdatePaymentResponse;
import rx.Single;

/**
 * Payment gateway public api. 200 - 300 http code is returned only if everything is successful.
 * <p>
 * If validation error occurs 422 code is returned.
 * Provide valid auth_token in {@link PgwModule} to avoid 401 errors.
 */
public interface PgwApi {
    /**
     * If error occurs while executing transaction proxy will returns {@link TransactionError}
     * <p>
     * To retrieve details about error invoke this method.
     *
     * @param id error id
     * @return Error details for id
     */
    Single<TransactionErrorDetailed> getError(int id);

    /**
     * Returns transaction details, status approval code etc.
     *
     * @param id Transaction id returned from payment gateway. See {@link TransactionSuccess#id}
     * @return Observable (single) of {@link Transaction}
     */
    Single<Transaction> checkTransaction(int id);

    /**
     * @param orderNumber order number
     * @param digest      sha1(merchant_key + orderNumber) -> Merchant key is provided on ipg merchant dashboard
     * @return ShowOrderResponse for orderNumber
     * @deprecated Deprecated in favor {@link #checkOrder(String, String)}
     */
    Single<ShowOrderResponse> checkOrderId(String orderNumber, String digest);


    /**
     * @param orderNumber order number
     * @param digest      sha1(merchant_key + orderNumber) -> Merchant key is provided on ipg merchant dashboard
     * @return ShowOrderResponse for orderNumber
     */
    Single<TransactionSuccess> checkOrder(String orderNumber, String digest);

    Single<PaymentMethodResponse> paymentMethods(String email);

    Single<DeletePaymentResponse> deletePaymentMethod(String paymentMethodId, String reason);

    Single<UpdatePaymentResponse> updatePaymentMethod(String paymentMethodId, Date expiryDate);

    Single<TransactionSuccess> charge(Charge charge);


}
