package hr.webtech.pay2.api;

import android.support.annotation.Nullable;

import com.google.gson.Gson;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import hr.webtech.pay2.data.Charge;
import hr.webtech.pay2.data.DeletePaymentMethodError;
import hr.webtech.pay2.data.DeletePaymentMethodSuccess;
import hr.webtech.pay2.data.DeletePaymentResponse;
import hr.webtech.pay2.data.PaymentMethodResponse;
import hr.webtech.pay2.data.ShowOrderResponse;
import hr.webtech.pay2.data.Transaction;
import hr.webtech.pay2.data.TransactionErrorDetailed;
import hr.webtech.pay2.data.TransactionSuccess;
import hr.webtech.pay2.data.UpdatePaymentMethodError;
import hr.webtech.pay2.data.UpdatePaymentMethodSuccess;
import hr.webtech.pay2.data.UpdatePaymentResponse;
import hr.webtech.pay2.util.Objects;
import retrofit2.HttpException;
import rx.Single;
import rx.functions.Func1;

/**
 * Created by jasminsuljic on 01/08/2018.
 * pgw-android
 */
class PgwApiProxy implements PgwApi {
    private final PgwRestApi pgwRestApi;
    private final Gson gson;

    PgwApiProxy(PgwRestApi pgwRestApi, Gson gson) {
        this.pgwRestApi = pgwRestApi;
        this.gson = gson;
    }

    @Override
    public Single<TransactionErrorDetailed> getError(int id) {
        return pgwRestApi.getError(id);
    }

    @Override
    public Single<Transaction> checkTransaction(int id) {
        return pgwRestApi.checkTransaction(id);
    }

    @Override
    public Single<ShowOrderResponse> checkOrderId(String orderNumber, String digest) {
        return pgwRestApi.checkOrderId(orderNumber, digest);
    }

    @Override
    public Single<TransactionSuccess> checkOrder(String orderNumber, String digest) {
        return pgwRestApi.checkOrder(orderNumber, digest);
    }

    @Override
    public Single<PaymentMethodResponse> paymentMethods(String email) {
        return pgwRestApi.paymentMethods(email);
    }

    @Override
    public Single<DeletePaymentResponse> deletePaymentMethod(String paymentMethodId, String reason) {

        return pgwRestApi.deletePaymentMethod(paymentMethodId, reason)
                .map(new Func1<DeletePaymentMethodSuccess, DeletePaymentResponse>() {
                    @Override
                    public DeletePaymentResponse call(DeletePaymentMethodSuccess deletePaymentMethodSuccess) {
                        return DeletePaymentResponse.create(deletePaymentMethodSuccess);
                    }
                })
                .onErrorReturn(new Func1<Throwable, DeletePaymentResponse>() {
                    @Override
                    public DeletePaymentResponse call(Throwable throwable) {

                        DeletePaymentMethodError deletePaymentMethodError = null;

                        if (throwable instanceof HttpException) {
                            deletePaymentMethodError = fromThrowable((HttpException) throwable);
                        }

                        return DeletePaymentResponse.create(
                                deletePaymentMethodError == null ?
                                        new DeletePaymentMethodError("not_deleted", throwable.getMessage(), new ArrayList<String>())
                                        : deletePaymentMethodError
                        );
                    }
                });
    }

    static String toExpiryDate(Date date) {
        Calendar cal = Calendar.getInstance();
        cal.setTime(date);
        int year = cal.get(Calendar.YEAR);
        int month = cal.get(Calendar.MONTH);
        return String.format("%d-%02d", year, month + 1);
    }

    @Override
    public Single<UpdatePaymentResponse> updatePaymentMethod(final String paymentMethodId, final Date expiryDate) {


        return Single.just(1)
                .flatMap(new Func1<Integer, Single<UpdatePaymentResponse>>() {
                    @Override
                    public Single<UpdatePaymentResponse> call(Integer integer) {

                        Objects.requireNonNull(paymentMethodId, "paymentMethodId == null");
                        Objects.requireNonNull(expiryDate, "expiryDate == null");
                        return pgwRestApi.updatePaymentMethod(paymentMethodId, toExpiryDate(expiryDate))
                                .map(new Func1<UpdatePaymentMethodSuccess, UpdatePaymentResponse>() {
                                    @Override
                                    public UpdatePaymentResponse call(UpdatePaymentMethodSuccess updatePaymentMethodSuccess) {
                                        return UpdatePaymentResponse.create(updatePaymentMethodSuccess);
                                    }
                                }).onErrorReturn(new Func1<Throwable, UpdatePaymentResponse>() {
                                    @Override
                                    public UpdatePaymentResponse call(Throwable throwable) {

                                        UpdatePaymentMethodError updatePaymentMethodError = null;

                                        if (throwable instanceof HttpException) {
                                            updatePaymentMethodError = updateErrorFromThrowable((HttpException) throwable);
                                        }
                                        return UpdatePaymentResponse.create(
                                                updatePaymentMethodError == null ?
                                                        new UpdatePaymentMethodError("not_updated", throwable.getMessage(), new ArrayList<String>())
                                                        : updatePaymentMethodError
                                        );
                                    }
                                });
                    }
                });
    }

    @Override
    public Single<TransactionSuccess> charge(Charge charge) {
        return this.pgwRestApi.charge(charge);
    }

    @Nullable
    private DeletePaymentMethodError fromThrowable(HttpException httpException) {
        try {
            final String bodyAsString = httpException.response().errorBody().string();
            return gson.fromJson(bodyAsString, DeletePaymentMethodError.class);
        } catch (Exception e) {
            return null;
        }
    }

    @Nullable
    private UpdatePaymentMethodError updateErrorFromThrowable(HttpException httpException) {
        try {
            final String bodyAsString = httpException.response().errorBody().string();
            return gson.fromJson(bodyAsString, UpdatePaymentMethodError.class);
        } catch (Exception e) {
            return null;
        }
    }
}
