package hr.webtech.pay2.api;

import hr.webtech.pay2.data.Charge;
import hr.webtech.pay2.data.DeletePaymentMethodSuccess;
import hr.webtech.pay2.data.PaymentMethodResponse;
import hr.webtech.pay2.data.ShowOrderResponse;
import hr.webtech.pay2.data.Transaction;
import hr.webtech.pay2.data.TransactionError;
import hr.webtech.pay2.data.TransactionErrorDetailed;
import hr.webtech.pay2.data.TransactionSuccess;
import hr.webtech.pay2.data.UpdatePaymentMethodSuccess;
import retrofit2.http.Body;
import retrofit2.http.Field;
import retrofit2.http.FormUrlEncoded;
import retrofit2.http.GET;
import retrofit2.http.POST;
import retrofit2.http.Path;
import retrofit2.http.Query;
import rx.Single;

/**
 * Created by jasminsuljic on 01/08/2018.
 * pgw-android
 */
interface PgwRestApi {
    /**
     * If error occurs while executing transaction proxy will returns {@link TransactionError}
     * <p>
     * To retrieve details about error invoke this method.
     *
     * @param id error id
     * @return Error details for id
     */
    @Deprecated
    @GET("get-error/{id}")
    Single<TransactionErrorDetailed> getError(@Path("id") int id);

    /**
     * Returns transaction details, status approval code etc.
     *
     * @param id Transaction id returned from payment gateway. See {@link TransactionSuccess#id}
     * @return Observable (single) of {@link Transaction}
     */
    @GET("check-transaction-id/{id}")
    Single<Transaction> checkTransaction(@Path("id") int id);

    /**
     * @param orderNumber order number
     * @param digest      sha1(merchant_key + orderNumber) -> Merchant key is provided on ipg merchant dashboard
     * @return ShowOrderResponse for orderNumber
     * @deprecated Deprecated in favor {@link #checkOrder(String, String)}
     */
    @POST("check-order-id")
    @FormUrlEncoded
    @Deprecated
    Single<ShowOrderResponse> checkOrderId(@Field("order_number") String orderNumber, @Field("digest") String digest);


    /**
     * @param orderNumber order number
     * @param digest      sha1(merchant_key + orderNumber) -> Merchant key is provided on ipg merchant dashboard
     * @return ShowOrderResponse for orderNumber
     */
    @POST("check-order")
    @FormUrlEncoded
    Single<TransactionSuccess> checkOrder(@Field("order_number") String orderNumber, @Field("digest") String digest);


    @GET("payment-methods/list")
    Single<PaymentMethodResponse> paymentMethods(@Query("id") String email);


    @POST("payment-method/delete")
    @FormUrlEncoded
    Single<DeletePaymentMethodSuccess> deletePaymentMethod(@Query("id") String paymentMethodId, @Field("delete_reason") String deleteReason);


    @POST("payment-method/update")
    @FormUrlEncoded
    Single<UpdatePaymentMethodSuccess> updatePaymentMethod(@Query("id") final String paymentMethodId, @Field("expiry_date") final String expiryDate);

    @POST("charge")
    Single<TransactionSuccess> charge(@Body Charge charge);
}
