package hr.webtech.pay2.api;

import java.io.IOException;
import java.lang.annotation.Annotation;

import okhttp3.ResponseBody;
import retrofit2.Converter;
import retrofit2.Retrofit;

/**
 * Created by jasminsuljic on 19/12/16.
 * pgw-android
 */
class RetrofitException extends RuntimeException {

    private final Kind kind;

    RetrofitException(String message, Kind kind, Throwable exception) {
        super(message, exception);
        this.kind = kind;
    }

    static <T> T getErrorBodyAs(Class<T> type, ResponseBody errorBody, Retrofit retrofit) throws IOException {
        if (errorBody == null) {
            return null;
        }
        Converter<ResponseBody, T> converter = retrofit.responseBodyConverter(type, new Annotation[0]);
        return converter.convert(errorBody);
    }

    /**
     * The event kind which triggered this error.
     */
    public Kind getKind() {
        return kind;
    }

    /**
     * Identifies the event kind which triggered a {@link RetrofitException}.
     */
    enum Kind {
        /**
         * An {@link IOException} occurred while communicating to the server.
         */
        NETWORK,
        /**
         * A non-200 HTTP status code was received from the server.
         */
        HTTP,
        API_ERROR_EXCEPTION, /**
         * An internal error occurred while attempting to execute a request. It is best practice to
         * re-throw this exception so your application crashes.
         */
        UNEXPECTED
    }
}
