package hr.webtech.pay2.api;

/**
 * Created by jasminsuljic on 19/12/16.
 * pgw-android
 */

import java.io.IOException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import hr.webtech.pay2.data.ApiError;
import hr.webtech.pay2.data.ApiErrors;
import hr.webtech.pay2.data.BusinessError;
import okhttp3.ResponseBody;
import retrofit2.Call;
import retrofit2.CallAdapter;
import retrofit2.Response;
import retrofit2.Retrofit;
import retrofit2.adapter.rxjava.HttpException;
import retrofit2.adapter.rxjava.RxJavaCallAdapterFactory;
import rx.Single;
import rx.functions.Func1;

import static hr.webtech.pay2.api.RetrofitException.getErrorBodyAs;

/**
 * Created by jasminsuljic on 16/07/16.
 * pgw-android
 * http://bytes.babbel.com/en/articles/2016-03-16-retrofit2-rxjava-error-handling.html
 */
final class RxErrorHandlingCallAdapterFactory extends CallAdapter.Factory {

    private final RxJavaCallAdapterFactory original;

    private RxErrorHandlingCallAdapterFactory() {
        original = RxJavaCallAdapterFactory.create();
    }

    static RxErrorHandlingCallAdapterFactory create() {
        return new RxErrorHandlingCallAdapterFactory();
    }

    @Override
    public CallAdapter<?, ?> get(Type returnType, Annotation[] annotations, Retrofit retrofit) {
        return new RxCallAdapterWrapper(retrofit, original.get(returnType, annotations, retrofit));
    }

    private static class RxCallAdapterWrapper implements CallAdapter<Single<?>, Single<?>> {
        private final Retrofit retrofit;
        private final CallAdapter<?, ?> wrapped;

        RxCallAdapterWrapper(Retrofit retrofit, CallAdapter<?, ?> wrapped) {
            this.retrofit = retrofit;
            this.wrapped = wrapped;
        }

        @Override
        public Type responseType() {
            return wrapped.responseType();
        }


        @SuppressWarnings("unchecked")
        @Override
        public Single<?> adapt(Call call) {
            return ((Single) wrapped.adapt(call)).onErrorResumeNext(new Func1<Throwable, Single>() {
                @Override
                public Single call(Throwable throwable) {
                    return Single.error(asApiError(throwable));
                }
            });
        }

        private ApiError asApiError(Throwable throwable) {
            // We had non-200 http error
            if (throwable instanceof HttpException) {
                HttpException httpException = (HttpException) throwable;
                Response response = httpException.response();

                try {
                    final ResponseBody errorBody = response == null ? null : response.errorBody();
                    final BusinessError businessError = getErrorBodyAs(BusinessError.class, errorBody, retrofit);

                    if (businessError != null) {
                        return businessError.isValid() ? businessError : businessError.asApiError();
                    }

                } catch (Throwable ignored) {
                }

                return ApiErrors.libraryHttpError(throwable);

            }
            // A network error happened
            if (throwable instanceof IOException) {
                return ApiErrors.libraryNetworkError(throwable);
            }

            // We don't know what happened. We need to simply convert to an unknown error

            return ApiErrors.libraryUnexpectedError(throwable);
        }

    }
}

