
package hr.webtech.pay2.data;

import android.support.annotation.NonNull;
import android.support.annotation.Nullable;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

import hr.webtech.pay2.util.Utils;

import static hr.webtech.pay2.util.Objects.requireNonNull;


/**
 * AddPaymentMethodModel
 * <p>
 */
public final class AddPaymentMethodModel {

    /**
     * Handlebars template that will be loaded when add payment method is invoked
     * Default value is add_payment_method
     */
    @SerializedName("client")
    @Expose
    String client;
    /**
     * (Required)
     */
    @SerializedName("merchant_id")
    @Expose
    String merchantId;

    /**
     * (Required)
     */
    @SerializedName("client_secret")
    @Expose
    String clientSecret;
    /**
     * short description of order being processed
     */
    @SerializedName("order_info")
    @Expose
    String orderInfo;
    /**
     * unique identifier
     * (Required)
     */
    @SerializedName("order_number")
    @Expose
    String orderNumber;

    /**
     * unique identifier
     * (Required)
     */
    @SerializedName("sign")
    @Expose
    String sign;

    /**
     * (Required)
     */
    @SerializedName("auth_token")
    @Expose
    String authToken;

    @SerializedName("digest")
    @Expose
    String digest;


    @SerializedName("address")
    @Expose
    @Nullable
    String address;

    @SerializedName("phone_number")
    @Expose
    @Nullable
    String phoneNumber;

    @SerializedName("city")
    @Expose
    @Nullable
    String city;

    @SerializedName("email")
    @Expose
    @Nullable
    String email;

    @SerializedName("card_holder")
    @Expose
    @Nullable
    String cardHolder;

    @SerializedName("user_id")
    @Expose
    @Nullable
    String userId;


    /**
     * @param orderNumber
     * @param orderInfo
     * @param client
     * @param merchant
     * @param customer
     */
    public AddPaymentMethodModel(String orderNumber,
                                 String orderInfo,
                                 String sign,
                                 @Nullable String client,
                                 @NonNull Merchant merchant,
                                 @Nullable Customer customer
    ) {

        requireNonNull(merchant);

        this.orderNumber = requireNonNull(orderNumber);
        this.orderInfo = requireNonNull(orderInfo);
        this.sign = requireNonNull(sign);
        this.client = client == null ? "add_payment_method" : client;

        this.authToken = merchant.getAuthenticationToken();
        this.merchantId = merchant.getMerchantId();
        this.clientSecret = merchant.getClientSecret();

        if (customer == null) {
            this.address = null;
            this.phoneNumber = null;
            this.city = null;
            this.email = null;
            this.cardHolder = null;
            this.userId = null;
        } else {
            this.address = customer.getAddress();
            this.phoneNumber = customer.getPhoneNumber();
            this.city = customer.getCity();
            this.email = customer.getEmail();
            this.cardHolder = customer.getCardHolder();
            this.userId = customer.getUserId();
        }

        this.digest = generateDigest(merchant);
    }

    private String generateDigest(Merchant merchant) {

        String sb1 = merchant.getMerchantId() +
                orderNumber +
                merchant.getAuthenticationToken() +
                merchant.getClientSecret();
        return Utils.sha1Hash(sb1);
    }

    public String getOrderNumber() {
        return orderNumber;
    }
}