package hr.webtech.pay2.data;

import com.google.gson.annotations.Expose;
import com.google.gson.annotations.SerializedName;

import hr.webtech.pay2.util.Objects;


/**
 * Created by jasminsuljic on 30/11/2018.
 * pgw-android
 */
public final class Charge {
    @SerializedName("source")
    @Expose
    String source;

    @SerializedName("amount")
    @Expose
    String amount;

    @SerializedName("currency")
    @Expose
    String currency;

    @SerializedName("order_number")
    @Expose
    String orderNumber;

    @SerializedName("order_info")
    @Expose
    String orderInfo;

    public Charge() {
    }

    public Charge(String source, String amount, String currency, String orderNumber, String orderInfo) {
        this.source = source;
        this.amount = amount;
        this.currency = currency;
        this.orderNumber = orderNumber;
        this.orderInfo = orderInfo;
    }

    public static Charge create(PaymentMethod paymentMethod,
                                String amount,
                                AddCardModel.Currency currency,
                                String orderNumber,
                                String orderInfo) {
        Objects.requireNonNull(paymentMethod);
        Objects.requireNonNull(amount);
        Objects.requireNonNull(currency);
        Objects.requireNonNull(orderNumber);
        Objects.requireNonNull(orderInfo);

        return new Charge(
                paymentMethod.getId(),
                amount,
                currency.value,
                orderNumber,
                orderInfo
        );
    }

    public Charge setSource(String source) {
        this.source = source;
        return this;
    }

    public Charge setAmount(String amount) {
        this.amount = amount;
        return this;
    }

    public Charge setCurrency(String currency) {
        this.currency = currency;
        return this;
    }

    public Charge setOrderNumber(String orderNumber) {
        this.orderNumber = orderNumber;
        return this;
    }

    public Charge setOrderInfo(String orderInfo) {
        this.orderInfo = orderInfo;
        return this;
    }

    public String getSource() {

        return source;
    }

    public String getAmount() {
        return amount;
    }

    public String getCurrency() {
        return currency;
    }

    public String getOrderNumber() {
        return orderNumber;
    }

    public String getOrderInfo() {
        return orderInfo;
    }
}
