package hr.webtech.pay2.view;

import android.support.annotation.NonNull;

import hr.webtech.pay2.api.PgwApi;
import hr.webtech.pay2.util.Objects;
import rx.subscriptions.CompositeSubscription;

/**
 * Created by jasminsuljic on 20/12/16.
 * pgw-android
 */
public final class AddPaymentMethodLifecycleAdapter {
    /**
     * The Pgw api.
     */
    private final PgwApi pgwApi;
    /**
     * The Order number.
     */
    private final String orderNumber;
    /**
     * The Composite subscription.
     */
    private final CompositeSubscription compositeSubscription;
    /**
     * The Merchant key.
     */
    private final String merchantKey;

    @NonNull
    private final AddPaymentMethodLifecycleDelegate delegate;

    /**
     * @param orderNumber  order number
     * @param pgwApi       pgw api instance
     * @param merchantKey  merchant key, can be found under merchant profile -> api settings
     * @param delegate     {@link TransactionLifecycleDelegate} implementation
     */
    public AddPaymentMethodLifecycleAdapter(@NonNull String orderNumber, @NonNull PgwApi pgwApi, @NonNull String merchantKey, @NonNull AddPaymentMethodLifecycleDelegate delegate) {
        Objects.requireNonNull(orderNumber, "orderNumber == null");
        Objects.requireNonNull(pgwApi, "pgwApi == null");
        Objects.requireNonNull(merchantKey, "merchantKey == null");
        Objects.requireNonNull(delegate, "delegate == null");

        this.delegate = delegate;
        this.merchantKey = merchantKey;
        this.pgwApi = pgwApi;
        this.orderNumber = orderNumber;
        compositeSubscription = new CompositeSubscription();
    }

    /**
     * Invokes {@link TransactionLifecycleDelegate#stopWebViewLoad()} and checks current order status
     * via {@link PgwApi#checkOrderId(String, String)} after {@link WebtehWebView} loading timeout.
     */
    public final void timeout() {
        delegate.logTransactionInfo("TransactionLifecycleAdapter#timeout handling");

        delegate.logTransactionInfo("delegate#stopWebViewLoad");
        delegate.stopWebViewLoad();

        // TODO: fix timeout handling

//        Subscription subscribe = pgwApi.checkOrder(orderNumber, Utils.sha1Hash(merchantKey + orderNumber)).subscribeOn(Schedulers.io())
//                .observeOn(AndroidSchedulers.mainThread())
//                .subscribe(new Action1<TransactionSuccess>() {
//                    @Override
//                    public void call(TransactionSuccess transactionSuccess) {
//                        delegate.logTransactionInfo(String.format("Check order success %s", transactionSuccess));
//                        delegate.transactionSuccess(transactionSuccess);
//                    }
//                }, new Action1<Throwable>() {
//                    @Override
//                    public void call(Throwable throwable) {
//                        //noinspection ThrowableResultOfMethodCallIgnored
//                        final ApiError apiError = throwable instanceof ApiError ? (ApiError) throwable : ApiErrors.libraryUnknownError(throwable);
//                        delegate.logTransactionInfo(String.format("Check order failure %s", apiError));
//                        delegate.handleTransactionError(apiError);
//                    }
//                });
//
//        compositeSubscription.add(subscribe);
    }


    @SuppressWarnings("WeakerAccess")
    @NonNull
    public AddPaymentMethodLifecycleDelegate getDelegate() {
        return delegate;
    }

    void onDestroy() {
        compositeSubscription.clear();
    }


    /**
     * Describes type of handling used in {@link AddPaymentMethodLifecycleAdapter#timeout()} to resolve
     * transaction success.
     */
    @SuppressWarnings("WeakerAccess")
    public enum ResponseType {
        /**
         * {@link PgwApi#checkOrderId(String, String)} will be used
         */
        ShowOrderResponse,
        /**
         * {@link PgwApi#checkOrder(String, String)} will be used
         */
        TransactionSuccess
    }
}
