package hr.webtech.pay2.view;

import android.app.Activity;

import hr.webtech.pay2.data.ApiError;
import hr.webtech.pay2.data.WebtehErrorException;

/**
 * Created by jasminsuljic on 01/10/2018.
 * pgw-android
 */
interface SharedLifecycleDelegate {
    /**
     * Invoked after transaction is started.
     * <p>
     * Transaction is started in two cases:
     * 1. Regular non 3ds transaction
     * 2. After submitting 3ds form
     */
    void showProgress();

    /**
     * Hide progress bar. Invoked after:
     * 1. Transaction fails
     * 2. Transaction is started but redirected to 3ds page. Check {@link WebtehWebView.ResolvedUrlResult#STARTED_TRANSACTION_3DS}
     */
    void hideProgress();

    /**
     * Invoked if user press hardware back button while transaction is in progress.
     * Can be used to show additional info, alert dialog etc.
     * <p>
     * Requirements:
     * - override {@link Activity#onBackPressed()}
     * - invoke {@link WebtehWebView#isBackAllowed()}
     * - if navigation back is disabled {@link WebtehWebView#isBackAllowed()} will invoke {@link TransactionLifecycleDelegate#backPressedTransactionInProgress()}
     *
     * Call is blocking!
     */
    void backPressedTransactionInProgress();

    /**
     * Invoked after 3ds form is submitted
     *
     * @since 1.4.1
     */
    void threeDsSubmitted();

    /**
     * Invoked after 3ds form is loaded
     *
     * @since 1.4.1
     */
    void threeDsLoaded();

    /**
     * // TODO: 25/01/17 update docs
     * <p>
     * <p>
     * If apiError is instance of {@link WebtehErrorException} then error is in app error.
     *
     * @param apiError the apiError
     * @since 1.5.0
     */
    void handleTransactionError(ApiError apiError);

    /**
     * Invoked after {@link TransactionLifecycleAdapter#timeout()}, before timeout handling.
     * <p>
     * Implementation should invoke {@link WebtehWebView#stopLoading()}
     *
     * @since 1.4.1
     */
    void stopWebViewLoad();

    /**
     * This method is used to redirect logs from library to application.
     * <p>
     * Implement your own logging logic in this method
     *
     * @param loggingInfo
     */
    void logTransactionInfo(String loggingInfo);
}
