package hr.webtech.pay2.view;

import android.support.annotation.Nullable;

import com.google.gson.Gson;

import hr.webtech.pay2.data.TransactionSuccess;

/**
 * Created by jasminsuljic on 01/10/2018.
 * pgw-android
 */
class SuccessTransactionResolverRunnable implements RedirectResolver.RedirectResolverRunnable {

    private final TransactionLifecycleDelegate delegate;
    private final Gson gson;

    SuccessTransactionResolverRunnable(TransactionLifecycleDelegate delegate, Gson gson) {
        this.delegate = delegate;
        this.gson = gson;
    }

    @Override
    public void run(WebtehWebView.ResolvedUrlResult resolve, @Nullable String request, RedirectResolver redirectResolver, String inputParam) {
        // If transaction is not in progress there is no need for transaction success handling
        // This is guard from multiple delegate calls when timeout is invoked and webview is not stopped.
        // After a while transaction success or transaction failure occurs.
        if (!redirectResolver.isTransactionInProgress()) {
            delegate.logTransactionInfo("Transaction success invoked, but transactionInProgress = false. Ignoring.");
            return;
        }

        redirectResolver.setTransactionInProgress(false);
        redirectResolver.stopTimeoutTracking();

        delegate.hideProgress();
        final TransactionSuccess transactionSuccess = gson.fromJson(inputParam, TransactionSuccess.class);
        delegate.logTransactionInfo(String.format("Transaction success %s", transactionSuccess));
        delegate.transactionSuccess(transactionSuccess);
    }
}
